@@ -1,4 +1,4 @@
-/* $Id: SUPDrvTracer.cpp 102121 2015-08-14 15:16:38Z bird $ */
+/* $Id: SUPDrvTracer.cpp 104498 2015-12-03 21:42:04Z bird $ */
 /** @file
  * VBoxDrv - The VirtualBox Support Driver - Tracer Interface.
  */
@@ -248,7 +248,6 @@ static int supdrvVtgValidateString(const char *psz)
  * @param   pVtgHdr             The header.
  * @param   uVtgHdrAddr         The address where the header is actually
  *                              loaded.
- * @param   cbVtgObj            The alleged size of the header.
  * @param   pbImage             The image base, if available.
  * @param   cbImage             The image size, if available.
  * @param   fUmod               Whether this is a user module.
@@ -538,6 +537,8 @@ static int supdrvVtgValidate(PVTGOBJHDR pVtgHdr, RTUINTPTR uVtgHdrAddr, const ui
         MY_VALIDATE_ATTR(pProvider->AttrNames);
         MY_VALIDATE_ATTR(pProvider->AttrArguments);
         MY_CHECK_RET(pProvider->bReserved == 0, VERR_SUPDRV_VTG_BAD_PROVIDER);
+        MY_CHECK_RET(pProvider->cProbesEnabled == 0, VERR_SUPDRV_VTG_BAD_PROVIDER);
+        MY_CHECK_RET(pProvider->uSettingsSerialNo == 0, VERR_SUPDRV_VTG_BAD_PROVIDER);
     }
 
     /*
@@ -867,7 +868,7 @@ static void supdrvTracerRemoveAllProviders(PSUPDRVDEVEXT pDevExt)
  * Registers the VTG tracepoint providers of a driver.
  *
  * @returns VBox status code.
- * @param   pszName             The driver name.
+ * @param   pDevExt             The device instance data.
  * @param   pVtgHdr             The VTG object header.
  * @param   pImage              The image if applicable.
  * @param   pSession            The session if applicable.
@@ -1102,7 +1103,6 @@ SUPR0DECL(int) SUPR0TracerRegisterDrv(PSUPDRVSESSION pSession, PVTGOBJHDR pVtgHd
  * Deregister the VTG tracepoint providers of a driver.
  *
  * @param   pSession            The support driver session handle.
- * @param   pVtgHdr             The VTG header.
  */
 SUPR0DECL(void) SUPR0TracerDeregisterDrv(PSUPDRVSESSION pSession)
 {
