@@ -1,4 +1,4 @@
-/* $Id: intnetinline.h 103598 2015-10-22 12:17:19Z bird $ */
+/* $Id: intnetinline.h 105617 2016-02-19 23:18:18Z bird $ */
 /** @file
  * INTNET - Internal Networking, Inlined Code. (DEV,++)
  *
@@ -793,7 +793,7 @@ DECLINLINE(void) IntNetBufInit(PINTNETBUF pIntBuf, uint32_t cbBuf, uint32_t cbRe
     Assert(cbBuf >= sizeof(INTNETBUF) + cbRecv + cbSend);
     Assert(RT_ALIGN_32(cbRecv, INTNETRINGBUF_ALIGNMENT) == cbRecv);
     Assert(RT_ALIGN_32(cbSend, INTNETRINGBUF_ALIGNMENT) == cbSend);
-    Assert(ASMMemIsAll8(pIntBuf, cbBuf, '\0') == NULL);
+    Assert(ASMMemIsZero(pIntBuf, cbBuf));
 
     pIntBuf->u32Magic  = INTNETBUF_MAGIC;
     pIntBuf->cbBuf     = cbBuf;
